@@ -7,7 +7,7 @@ use crate::compiler::Compiler;
 use crate::error::{Error, ErrorKind};
 use crate::instructions::Instructions;
 use crate::parser::{parse, parse_expr};
-use crate::utils::{AutoEscape, HtmlEscape};
+use crate::utils::{AutoEscape, BTreeMapKeysDebug, HtmlEscape};
 use crate::value::{ArgType, FunctionArgs, RcType, Value};
 use crate::vm::Vm;
 use crate::{filters, functions, tests};
@@ -38,22 +38,33 @@ impl<'env> fmt::Debug for Template<'env> {
 }
 
 /// Represents a compiled template in memory.
-#[derive(Debug, Clone)]
+#[derive(Clone)]
 pub(crate) struct CompiledTemplate<'source> {
+    source: &'source str,
     instructions: Instructions<'source>,
     blocks: BTreeMap<&'source str, Instructions<'source>>,
 }
 
+impl<'env> fmt::Debug for CompiledTemplate<'env> {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.debug_struct("CompiledTemplate")
+            .field("instructions", &self.instructions)
+            .field("blocks", &self.blocks)
+            .finish()
+    }
+}
+
 impl<'source> CompiledTemplate<'source> {
     pub(crate) fn from_name_and_source(
-        name: &str,
+        name: &'source str,
         source: &'source str,
     ) -> Result<CompiledTemplate<'source>, Error> {
         let ast = parse(source, name)?;
-        let mut compiler = Compiler::new();
+        let mut compiler = Compiler::new(name);
         compiler.compile_stmt(&ast)?;
         let (instructions, blocks) = compiler.finish();
         Ok(CompiledTemplate {
+            source,
             blocks,
             instructions,
         })
@@ -66,6 +77,11 @@ impl<'env> Template<'env> {
         self.name
     }
 
+    /// Returns the source code of the template.
+    pub fn source(&self) -> &str {
+        self.compiled.source
+    }
+
     /// Renders the template into a string.
     ///
     /// The provided value is used as the initial context for the template.  It
@@ -73,12 +89,18 @@ impl<'env> Template<'env> {
     /// Typically custom structs annotated with `#[derive(Serialize)]` would
     /// be used for this purpose.
     pub fn render<S: Serialize>(&self, ctx: S) -> Result<String, Error> {
+        // reduce total amount of code faling under mono morphization into
+        // this function, and share the rest in _eval.
+        self._render(Value::from_serializable(&ctx))
+    }
+
+    fn _render(&self, root: Value) -> Result<String, Error> {
         let mut output = String::new();
         let vm = Vm::new(self.env);
         let blocks = &self.compiled.blocks;
         vm.eval(
             &self.compiled.instructions,
-            ctx,
+            root,
             blocks,
             self.initial_auto_escape,
             &mut output,
@@ -114,7 +136,7 @@ enum Source<'source> {
 impl<'source> fmt::Debug for Source<'source> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match self {
-            Self::Borrowed(arg0) => fmt::Debug::fmt(arg0, f),
+            Self::Borrowed(tmpls) => fmt::Debug::fmt(&BTreeMapKeysDebug(tmpls), f),
             #[cfg(feature = "source")]
             Self::Owned(arg0) => fmt::Debug::fmt(arg0, f),
         }
@@ -153,6 +175,9 @@ impl<'source> Default for Environment<'source> {
 impl<'source> fmt::Debug for Environment<'source> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         f.debug_struct("Environment")
+            .field("globals", &self.globals)
+            .field("tests", &BTreeMapKeysDebug(&self.tests))
+            .field("filters", &BTreeMapKeysDebug(&self.filters))
             .field("templates", &self.templates)
             .finish()
     }
@@ -177,16 +202,16 @@ fn no_auto_escape(_: &str) -> AutoEscape {
 /// This for instance can be used to evaluate simple expressions from user
 /// provided input to implement features such as dynamic filtering.
 ///
-/// This is usually best paired with [`Single`](crate::value::Single) to pass
+/// This is usually best paired with [`context`](crate::context!) to pass
 /// a single value to it.
 ///
 /// # Example
 ///
 /// ```rust
-/// # use minijinja::{Environment, value::Single};
+/// # use minijinja::{Environment, context};
 /// let env = Environment::new();
 /// let expr = env.compile_expression("number > 10 and number < 20").unwrap();
-/// let rv = expr.eval(Single("number", 15)).unwrap();
+/// let rv = expr.eval(context!(number => 15)).unwrap();
 /// assert!(rv.is_true());
 /// ```
 #[derive(Debug)]
@@ -200,13 +225,19 @@ impl<'env, 'source> Expression<'env, 'source> {
     ///
     /// The result of the expression is returned as [`Value`].
     pub fn eval<S: Serialize>(&self, ctx: S) -> Result<Value, Error> {
+        // reduce total amount of code faling under mono morphization into
+        // this function, and share the rest in _eval.
+        self._eval(Value::from_serializable(&ctx))
+    }
+
+    fn _eval(&self, root: Value) -> Result<Value, Error> {
         let mut output = String::new();
         let vm = Vm::new(self.env);
         let blocks = BTreeMap::new();
         Ok(vm
             .eval(
                 &self.instructions,
-                ctx,
+                root,
                 &blocks,
                 AutoEscape::None,
                 &mut output,
@@ -345,7 +376,7 @@ impl<'source> Environment<'source> {
     /// example see [`Expression`].
     pub fn compile_expression(&self, expr: &'source str) -> Result<Expression<'_, 'source>, Error> {
         let ast = parse_expr(expr)?;
-        let mut compiler = Compiler::new();
+        let mut compiler = Compiler::new("<expression>");
         compiler.compile_expr(&ast)?;
         let (instructions, _) = compiler.finish();
         Ok(Expression {
@@ -417,47 +448,25 @@ impl<'source> Environment<'source> {
         self.globals.get(name).cloned()
     }
 
-    /// Applies a filter with arguments to a value.
-    pub(crate) fn apply_filter(
-        &self,
-        name: &str,
-        value: Value,
-        args: Vec<Value>,
-    ) -> Result<Value, Error> {
-        if let Some(filter) = self.filters.get(name) {
-            filter.apply_to(self, value, args)
-        } else {
-            Err(Error::new(
-                ErrorKind::UnknownFilter,
-                format!("filter {} is unknown", name),
-            ))
-        }
+    /// Looks up a filter.
+    pub(crate) fn get_filter(&self, name: &str) -> Option<&filters::BoxedFilter> {
+        self.filters.get(name)
     }
 
-    /// Performs a test.
-    pub(crate) fn perform_test(
-        &self,
-        name: &str,
-        value: Value,
-        args: Vec<Value>,
-    ) -> Result<bool, Error> {
-        if let Some(test) = self.tests.get(name) {
-            test.perform(self, value, args)
-        } else {
-            Err(Error::new(
-                ErrorKind::UnknownTest,
-                format!("test {} is unknown", name),
-            ))
-        }
+    /// Looks up a test function.
+    pub(crate) fn get_test(&self, name: &str) -> Option<&tests::BoxedTest> {
+        self.tests.get(name)
     }
 
     /// Finalizes a value.
-    pub(crate) fn finalize<W: fmt::Write>(
+    pub(crate) fn finalize(
         &self,
         value: &Value,
         autoescape: AutoEscape,
-        out: &mut W,
+        out: &mut String,
     ) -> Result<(), Error> {
+        use std::fmt::Write;
+
         // safe values do not get escaped
         if value.is_safe() {
             write!(out, "{}", value).unwrap();
