@@ -10,7 +10,7 @@ fn test_loop() {
         Value::from((1..=9).into_iter().collect::<Vec<_>>()),
     );
 
-    let mut c = Compiler::new();
+    let mut c = Compiler::default();
     c.add(Instruction::Lookup("items"));
     c.start_for_loop(false);
     c.add(Instruction::Emit);
@@ -29,7 +29,7 @@ fn test_if() {
         let mut ctx = std::collections::BTreeMap::new();
         ctx.insert("cond", Value::from(val));
 
-        let mut c = Compiler::new();
+        let mut c = Compiler::default();
         c.add(Instruction::Lookup("cond"));
         c.start_if();
         c.add(Instruction::EmitRaw("true"));
@@ -50,7 +50,7 @@ fn test_if_branches() {
     ctx.insert("false", Value::from(false));
     ctx.insert("nil", Value::from(()));
 
-    let mut c = Compiler::new();
+    let mut c = Compiler::default();
     c.add(Instruction::Lookup("false"));
     c.start_if();
     c.add(Instruction::EmitRaw("nope1"));
@@ -98,8 +98,7 @@ fn test_basic() {
 
 #[test]
 fn test_error_info() {
-    let mut c = Compiler::new();
-    c.set_file("hello.html");
+    let mut c = Compiler::new("hello.html");
     c.set_line(1);
     c.add(Instruction::EmitRaw("<h1>Hello</h1>\n"));
     c.set_line(2);
@@ -118,7 +117,7 @@ fn test_error_info() {
 
 #[test]
 fn test_op_eq() {
-    let mut c = Compiler::new();
+    let mut c = Compiler::default();
     c.add(Instruction::LoadConst(Value::from(1)));
     c.add(Instruction::LoadConst(Value::from(1)));
     c.add(Instruction::Eq);
@@ -128,7 +127,7 @@ fn test_op_eq() {
     simple_eval(&c.finish().0, (), &mut output).unwrap();
     assert_eq!(output, "true");
 
-    let mut c = Compiler::new();
+    let mut c = Compiler::default();
     c.add(Instruction::LoadConst(Value::from(1)));
     c.add(Instruction::LoadConst(Value::from(2)));
     c.add(Instruction::Eq);
@@ -141,7 +140,7 @@ fn test_op_eq() {
 
 #[test]
 fn test_op_ne() {
-    let mut c = Compiler::new();
+    let mut c = Compiler::default();
     c.add(Instruction::LoadConst(Value::from(1)));
     c.add(Instruction::LoadConst(Value::from("foo")));
     c.add(Instruction::Ne);
@@ -151,7 +150,7 @@ fn test_op_ne() {
     simple_eval(&c.finish().0, (), &mut output).unwrap();
     assert_eq!(output, "true");
 
-    let mut c = Compiler::new();
+    let mut c = Compiler::default();
     c.add(Instruction::LoadConst(Value::from("foo")));
     c.add(Instruction::LoadConst(Value::from("foo")));
     c.add(Instruction::Ne);
@@ -164,7 +163,7 @@ fn test_op_ne() {
 
 #[test]
 fn test_op_lt() {
-    let mut c = Compiler::new();
+    let mut c = Compiler::default();
     c.add(Instruction::LoadConst(Value::from(1)));
     c.add(Instruction::LoadConst(Value::from(2)));
     c.add(Instruction::Lt);
@@ -174,7 +173,7 @@ fn test_op_lt() {
     simple_eval(&c.finish().0, (), &mut output).unwrap();
     assert_eq!(output, "true");
 
-    let mut c = Compiler::new();
+    let mut c = Compiler::default();
     c.add(Instruction::LoadConst(Value::from(2)));
     c.add(Instruction::LoadConst(Value::from(1)));
     c.add(Instruction::Lt);
@@ -187,7 +186,7 @@ fn test_op_lt() {
 
 #[test]
 fn test_op_gt() {
-    let mut c = Compiler::new();
+    let mut c = Compiler::default();
     c.add(Instruction::LoadConst(Value::from(1)));
     c.add(Instruction::LoadConst(Value::from(2)));
     c.add(Instruction::Gt);
@@ -197,7 +196,7 @@ fn test_op_gt() {
     simple_eval(&c.finish().0, (), &mut output).unwrap();
     assert_eq!(output, "false");
 
-    let mut c = Compiler::new();
+    let mut c = Compiler::default();
     c.add(Instruction::LoadConst(Value::from(2)));
     c.add(Instruction::LoadConst(Value::from(1)));
     c.add(Instruction::Gt);
@@ -210,7 +209,7 @@ fn test_op_gt() {
 
 #[test]
 fn test_op_lte() {
-    let mut c = Compiler::new();
+    let mut c = Compiler::default();
     c.add(Instruction::LoadConst(Value::from(1)));
     c.add(Instruction::LoadConst(Value::from(1)));
     c.add(Instruction::Lte);
@@ -220,7 +219,7 @@ fn test_op_lte() {
     simple_eval(&c.finish().0, (), &mut output).unwrap();
     assert_eq!(output, "true");
 
-    let mut c = Compiler::new();
+    let mut c = Compiler::default();
     c.add(Instruction::LoadConst(Value::from(2)));
     c.add(Instruction::LoadConst(Value::from(1)));
     c.add(Instruction::Lte);
@@ -233,7 +232,7 @@ fn test_op_lte() {
 
 #[test]
 fn test_op_gte() {
-    let mut c = Compiler::new();
+    let mut c = Compiler::default();
     c.add(Instruction::LoadConst(Value::from(1)));
     c.add(Instruction::LoadConst(Value::from(2)));
     c.add(Instruction::Gte);
@@ -243,7 +242,7 @@ fn test_op_gte() {
     simple_eval(&c.finish().0, (), &mut output).unwrap();
     assert_eq!(output, "false");
 
-    let mut c = Compiler::new();
+    let mut c = Compiler::default();
     c.add(Instruction::LoadConst(Value::from(1)));
     c.add(Instruction::LoadConst(Value::from(1)));
     c.add(Instruction::Gte);
@@ -256,7 +255,7 @@ fn test_op_gte() {
 
 #[test]
 fn test_op_not() {
-    let mut c = Compiler::new();
+    let mut c = Compiler::default();
     c.add(Instruction::LoadConst(Value::from(0)));
     c.add(Instruction::Not);
     c.add(Instruction::Emit);
@@ -265,7 +264,7 @@ fn test_op_not() {
     simple_eval(&c.finish().0, (), &mut output).unwrap();
     assert_eq!(output, "true");
 
-    let mut c = Compiler::new();
+    let mut c = Compiler::default();
     c.add(Instruction::LoadConst(Value::from(true)));
     c.add(Instruction::Not);
     c.add(Instruction::Emit);
@@ -277,7 +276,7 @@ fn test_op_not() {
 
 #[test]
 fn test_string_concat() {
-    let mut c = Compiler::new();
+    let mut c = Compiler::default();
     c.add(Instruction::LoadConst(Value::from("foo")));
     c.add(Instruction::LoadConst(Value::from(42)));
     c.add(Instruction::StringConcat);
@@ -290,7 +289,7 @@ fn test_string_concat() {
 
 #[test]
 fn test_unpacking() {
-    let mut c = Compiler::new();
+    let mut c = Compiler::default();
     c.add(Instruction::LoadConst(Value::from(vec!["bar", "foo"])));
     c.add(Instruction::UnpackList(2));
     c.add(Instruction::StringConcat);
